"use strict";
var parse = require('url').parse;
var request = require('superagent');
var Q = require('q');

module.exports.id = "itunes";

var apiRoot = "https://itunes.apple.com";

module.exports.match = function(url, type) {
  var parsed = parse(url);
  return parsed.host.match(/itunes.apple\.com$/);
};

module.exports.lookupId = function(id, type) {
  var deferred = Q.defer();
  var path = "/lookup?id=" + id;

  request.get(apiRoot + path, function(res) {
    var data = JSON.parse(res.text);

    if (!data.results[0].collectionId) {
      deferred.resolve({service: "itunes"});
    } else {
      var result = data.results[0];

      var item = {
        service: "itunes",
        type: type,
        id: result.collectionId,
        name: result.trackName ? result.trackName : result.collectionName,
        streamUrl: null,
        purchaseUrl: result.collectionViewUrl,
        artwork: result.artworkUrl100.replace("100x100", "200x200").replace("http:", ""),
        artist: {
          name: result.artistName
        }
      };

      if (type == "track") {
        item.album = {
          name: result.collectionName
        };
      }

      deferred.resolve(item);
    }
  });
  return deferred.promise;
}

module.exports.search = function(data) {
  var deferred = Q.defer();
  var query = "";
  var entity = "";
  var type = data.type;

  if (type == "album") {
    query = data.artist.name + " " + data.name;
    entity = "album";
  } else if (type == "track") {
    query = data.artist.name + " " + data.album.name + " " + data.name;
    entity = "musicTrack";
  }

  var path = "/search?term=" + encodeURIComponent(query) + "&media=music&entity=" + entity;
  request.get(apiRoot + path, function(res) {
    var data = JSON.parse(res.text);

    if (!data.results[0].collectionId) {
      deferred.resolve({service: "itunes"});
    } else {
      var result = data.results[0];

      var item = {
        service: "itunes",
        type: type,
        id: result.collectionId,
        name: result.trackName ? result.trackName : result.collectionName,
        streamUrl: null,
        purchaseUrl: result.collectionViewUrl,
        artwork: result.artworkUrl100.replace("100x100", "200x200").replace("http:", ""),
        artist: {
          name: result.artistName
        }
      };

      if (type == "track") {
        item.album = {
          name: result.collectionName
        };
      }
      deferred.resolve(item);
    }
  });

  return deferred.promise;
}

module.exports.parseUrl = function(url) {
  var deferred = Q.defer();
  var matches = parse(url).path.match(/\/(album|track)[\/]+([^\/]+)[\/]+([^\?]+)/);

  if (matches && matches[3]) {
    module.exports.lookupId(matches[3].substr(2), matches[1]).then(deferred.resolve);
  }
  return deferred.promise;
}
